package tekton

import (
	"net/http"

	"github.com/PingCAP-QE/ee-apps/cloudevents-server/pkg/config"
	cloudevents "github.com/cloudevents/sdk-go/v2"
	lark "github.com/larksuite/oapi-sdk-go/v3"
	"github.com/rs/zerolog/log"
	tektoncloudevent "github.com/tektoncd/pipeline/pkg/reconciler/events/cloudevent"
)

type taskRunHandler struct {
	config.Tekton
	LarkClient *lark.Client
}

func (h *taskRunHandler) SupportEventTypes() []string {
	return []string{
		string(tektoncloudevent.TaskRunFailedEventV1),
		string(tektoncloudevent.TaskRunRunningEventV1),
		string(tektoncloudevent.TaskRunStartedEventV1),
		string(tektoncloudevent.TaskRunSuccessfulEventV1),
		string(tektoncloudevent.TaskRunUnknownEventV1),
	}
}

func (h *taskRunHandler) Handle(event cloudevents.Event) cloudevents.Result {
	data := new(tektoncloudevent.TektonCloudEventData)
	if err := event.DataAs(&data); err != nil {
		return cloudevents.NewHTTPResult(http.StatusBadRequest, err.Error())
	}

	switch event.Type() {
	case string(tektoncloudevent.TaskRunFailedEventV1):
		var receivers []string
		// send notify to the trigger user if it's existed, else send to the receivers configurated by type.
		if receiver := getTriggerUser(data.TaskRun); receiver != "" {
			receivers = []string{receiver}
		} else {
			receivers = append(h.Receivers[defaultReceiversKey], h.Receivers[event.Type()]...)
		}

		return sendLarkMessages(h.LarkClient, receivers, event, h.DashboardBaseURL)
	default:
		log.Debug().Str("ce-type", event.Type()).Msg("skip notifing for the event type.")
		return cloudevents.ResultACK
	}
}
